/*****************************************************************************
 * media_descriptor.c: Libvlc API media descriport management
 *****************************************************************************
 * Copyright (C) 2007 the VideoLAN team
 * $Id$
 *
 * Authors: Pierre d'Herbemont <pdherbemont@videolan.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#include <vlc/libvlc.h>
#include <vlc_input.h>
#include <vlc_meta.h>

#include "libvlc_internal.h"


/**************************************************************************
 * Preparse if not already done (Private)
 **************************************************************************/
static void preparse_if_needed( libvlc_media_descriptor_t *p_md )
{
    /* XXX: need some locking here */
    if (!p_md->b_preparsed)
    {
        input_Preparse( p_md->p_libvlc_instance->p_libvlc_int,
                        p_md->p_input_item );
        p_md->b_preparsed = VLC_TRUE;
    }
}

/**************************************************************************
 * Create a new media descriptor object
 **************************************************************************/
libvlc_media_descriptor_t * libvlc_media_descriptor_new(
                                   libvlc_instance_t *p_instance,
                                   const char * psz_mrl,
                                   libvlc_exception_t *p_e )
{
    input_item_t * p_input_item;
    libvlc_media_descriptor_t * p_md;

    p_input_item = input_ItemNew( p_instance->p_libvlc_int, psz_mrl, psz_mrl );

    if (!p_input_item)
    {
        libvlc_exception_raise( p_e, "Can't create md's input_item" );
        return NULL;
    }

    p_md = malloc( sizeof(libvlc_media_descriptor_t) );
    p_md->p_libvlc_instance = p_instance;
    p_md->p_input_item      = p_input_item;
    p_md->b_preparsed       = VLC_FALSE;
    p_md->i_refcount        = 1;
 
    vlc_gc_incref( p_md->p_input_item );

    return p_md;
}

/**************************************************************************
 * Create a new media descriptor object from an input_item
 * (libvlc internal)
 **************************************************************************/
libvlc_media_descriptor_t * libvlc_media_descriptor_new_from_input_item(
                                   libvlc_instance_t *p_instance,
                                   input_item_t *p_input_item,
                                   libvlc_exception_t *p_e )
{
    libvlc_media_descriptor_t * p_md;

    if (!p_input_item)
    {
        libvlc_exception_raise( p_e, "No input item given" );
        return NULL;
    }

    p_md = malloc( sizeof(libvlc_media_descriptor_t) );
    p_md->p_libvlc_instance = p_instance;
    p_md->p_input_item      = p_input_item;
    p_md->b_preparsed       = VLC_TRUE;
    p_md->i_refcount        = 1;

    vlc_gc_incref( p_md->p_input_item );

    return p_md;
}

/**************************************************************************
 * Delete a media descriptor object
 **************************************************************************/
void libvlc_media_descriptor_release( libvlc_media_descriptor_t *p_md )
{
    if (!p_md)
        return;

    p_md->i_refcount--;

    /* XXX: locking */
    vlc_gc_decref( p_md->p_input_item );

    if( p_md->i_refcount > 0 )
        return;
    
    free( p_md );
}

/**************************************************************************
 * Retain a media descriptor object
 **************************************************************************/
void libvlc_media_descriptor_retain( libvlc_media_descriptor_t *p_md )
{
    if (!p_md)
        return;

    p_md->i_refcount++;

    /* XXX: locking */
    vlc_gc_incref( p_md->p_input_item );
}

/**************************************************************************
 * Duplicate a media descriptor object
 **************************************************************************/
libvlc_media_descriptor_t *
libvlc_media_descriptor_duplicate( libvlc_media_descriptor_t *p_md_orig )
{
    libvlc_media_descriptor_t * p_md;

    p_md = malloc( sizeof(libvlc_media_descriptor_t) );
    memcpy( p_md, p_md_orig, sizeof(libvlc_media_descriptor_t) );

    vlc_gc_incref( p_md->p_input_item );

    return p_md;
}

/**************************************************************************
 * Retain a media descriptor object
 **************************************************************************/
char *
libvlc_media_descriptor_get_mrl( libvlc_media_descriptor_t * p_md,
                                 libvlc_exception_t * p_e )
{
    (void)p_e;
    return strdup( p_md->p_input_item->psz_uri );
}

/**************************************************************************
 * Getter for meta information
 **************************************************************************/
static const vlc_meta_type_t meta_conversion[] =
{
    [libvlc_meta_Artist]       = vlc_meta_Artist,
    [libvlc_meta_Genre]        = vlc_meta_Genre,
    [libvlc_meta_Copyright]    = vlc_meta_Copyright,
    [libvlc_meta_Album]        = vlc_meta_Album,
    [libvlc_meta_TrackNumber]  = vlc_meta_TrackNumber,
    [libvlc_meta_Description]  = vlc_meta_Description,
    [libvlc_meta_Rating]       = vlc_meta_Rating,
    [libvlc_meta_Date]         = vlc_meta_Date,
    [libvlc_meta_Settings]     = vlc_meta_Setting,
    [libvlc_meta_URL]          = vlc_meta_URL,
    [libvlc_meta_Language]     = vlc_meta_Language,
    [libvlc_meta_NowPlaying]   = vlc_meta_NowPlaying,
    [libvlc_meta_Publisher]    = vlc_meta_Publisher,
    [libvlc_meta_EncodedBy]    = vlc_meta_EncodedBy,
    [libvlc_meta_ArtworkURL]   = vlc_meta_ArtworkURL,
    [libvlc_meta_TrackID]      = vlc_meta_TrackID
};

char * libvlc_media_descriptor_get_meta( libvlc_media_descriptor_t *p_md,
                                         libvlc_meta_t e_meta,
                                         libvlc_exception_t *p_e )
{
    const char * psz_meta;

    /* XXX: locking */

    preparse_if_needed( p_md );

    psz_meta = input_item_GetMeta( p_md->p_input_item, meta_conversion[e_meta] );

    if( !psz_meta )
        return NULL;

    return strdup( psz_meta );
}

